#!/usr/bin/env python
#
#  _abc.py
"""
Abstract Base Class for JSON and YAML serializers.
"""
#
#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#  Based on Slumber <https://slumber.readthedocs.io>
#  Copyright (c) 2011 Donald Stufft
#  Licensed under the 2-clause BSD License
#

# stdlib
from abc import ABC, abstractmethod
from typing import Any, List, Mapping, MutableMapping

__all__ = ["Serializer"]


class Serializer(ABC):
	"""
	Base class for serializers.

	.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`
	"""

	@property
	@abstractmethod
	def content_types(self) -> List[str]:  # pragma: no cover
		"""
		List of supported content types.
		"""

		return NotImplemented

	@property
	@abstractmethod
	def key(self) -> str:  # pragma: no cover
		"""
		An identifier for the supported data type.

		For example, a YAML serializer would set this to ``'yaml'``.
		"""

		return NotImplemented

	def get_content_type(self) -> str:
		"""
		Returns the first value from :attr:`~.Serializer.content_types`.
		"""

		return self.content_types[0]

	@abstractmethod
	def loads(self, data: str) -> MutableMapping[str, Any]:
		"""
		Deserialize data using this :class:`~.Serializer`.

		:param data:
		"""

		raise NotImplementedError()

	@abstractmethod
	def dumps(self, data: Mapping[str, Any]) -> str:
		"""
		Serialize data using this :class:`~.Serializer`.

		:param data:
		"""

		raise NotImplementedError()



#!/usr/bin/env python3
#
#  cache.py
"""
Caching functions for functions.

.. latex:vspace:: 10px

.. seealso::

	* The `cachier project <https://pypi.org/project/cachier/>`_
	* `DiskCache <http://www.grantjenks.com/docs/diskcache/index.html>`_

.. automodulesumm:: apeye.cache
	:autosummary-sections: ;;
"""
#
#  Copyright (c) 2020 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# stdlib
import inspect
import json
import shutil
import warnings
from functools import wraps
from typing import Any, Callable, Dict, Iterable, Optional

# 3rd party
import platformdirs
from domdf_python_tools.paths import PathPlus
from domdf_python_tools.utils import posargs2kwargs

__all__ = ["Cache"]


class Cache:
	"""
	Cache function arguments to and in-memory dictionary and a JSON file.

	:param app_name: The name of the app. This dictates the name of the cache directory.
	"""

	app_name: str  #: The name of the app. This dictates the name of the cache directory.
	cache_dir: PathPlus  #: The location of the cache directory on disk.
	caches: Dict[str, Dict[str, Any]]  #: Mapping of function names to their caches.

	def __init__(self, app_name: str):
		self.app_name: str = str(app_name)
		self.cache_dir = PathPlus(platformdirs.user_cache_dir(f"{self.app_name}_cache"))
		self.cache_dir.maybe_make(parents=True)

		# Mapping of function names to their caches
		self.caches: Dict[str, Dict[str, Any]] = {}

	def clear(self, func: Optional[Callable] = None) -> bool:
		"""
		Clear the cache.

		:param func: Optional function to clear the cache for.
			By default, the whole cache is cleared.
		:no-default func:

		:returns: True to indicate success. False otherwise.
		"""

		try:
			if func is None:
				shutil.rmtree(self.cache_dir)
				self.cache_dir.maybe_make()
				for function in self.caches:
					self.caches[function] = {}
			else:
				function_name = func.__name__
				cache_file = self.cache_dir / f"{function_name}.json"
				if cache_file.is_file():
					cache_file.unlink()

				if function_name in self.caches:
					del self.caches[function_name]
				self.caches[function_name] = {}

			return True

		except Exception as e:  # pragma: no cover
			warnings.warn(f"Could not remove cache. The error was: {e}")
			return False

	def load_cache(self, func: Callable) -> None:
		"""
		Loads the cache for the given function.

		:param func:
		"""

		cache_file: PathPlus = self.cache_dir / f"{func.__name__}.json"

		if cache_file.is_file():
			cache = json.loads(cache_file.read_text())
		else:
			cache = {}

		self.caches[func.__name__] = cache
		return cache

	def __call__(self, func: Callable):
		"""
		Decorator to cache the return values of a function based on its inputs.

		:param func:
		"""

		function_name = func.__name__
		posargs: Iterable[str] = inspect.getfullargspec(func).args
		cache_file: PathPlus = self.cache_dir / f"{function_name}.json"
		self.load_cache(func)

		@wraps(func)
		def wrapper(*args, **kwargs: Any):
			kwargs: Dict[str, Any] = posargs2kwargs(args, posargs, kwargs)  # type: ignore
			key: str = json.dumps(kwargs)
			response: Any

			cache = self.caches[function_name]
			if key in cache:
				# Return cached response
				response = json.loads(cache[key])
			else:
				response = func(**kwargs)

				if response is not None:
					# Don't cache None values.
					cache[key] = json.dumps(response)

			cache_file.write_text(json.dumps(cache))

			return response

		return wrapper



#!/usr/bin/env python
#
#  __init__.py
"""
JSON and YAML serializers for :class:`~apeye.slumber_url.SlumberURL`.

.. versionadded:: 0.6.0
"""
#
#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#  Based on Slumber <https://slumber.readthedocs.io>
#  Copyright (c) 2011 Donald Stufft
#  Licensed under the 2-clause BSD License
#

# stdlib
import json
from typing import Any, Dict, List, Mapping, MutableMapping, Optional, Type

# this package
from apeye.slumber_url.exceptions import SlumberBaseException
from apeye.slumber_url.serializers._abc import Serializer

__all__ = ["JsonSerializer", "YamlSerializer", "SerializerRegistry", "Serializer", "SerializerNotAvailable"]


class JsonSerializer(Serializer):
	"""
	Serializer for JSON data.

	.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`
	"""

	content_types = [
			"application/json",
			"application/x-javascript",
			"text/javascript",
			"text/x-javascript",
			"text/x-json",
			]
	key = "json"

	def loads(self, data: str) -> MutableMapping[str, Any]:
		"""
		Deserialize data using this :class:`~.Serializer`.

		:param data:
		"""

		return json.loads(data)

	def dumps(self, data: Mapping[str, Any]) -> str:
		"""
		Serialize data using this :class:`~.Serializer`.

		:param data:
		"""

		return json.dumps(data)


_SERIALIZERS: List[Type[Serializer]] = [JsonSerializer]

YamlSerializer: Type[Serializer]

try:
	# this package
	from apeye.slumber_url.serializers._pyyaml_serializer import YamlSerializer
	_SERIALIZERS.append(YamlSerializer)

except ImportError:
	try:
		# this package
		from apeye.slumber_url.serializers._ruamel_serializer import YamlSerializer
		_SERIALIZERS.append(YamlSerializer)

	except ImportError:

		class YamlSerializer(Serializer):  # type: ignore[no-redef]
			"""
			Serializer for YAML data.

			.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`

			.. attention::

				Either `PyYaml <https://pypi.org/project/PyYAML/>`_ or
				`ruamel.yaml <https://pypi.org/project/ruamel.yaml/>`_
				must be installed to use this serializer.
			"""

			content_types = ["text/yaml"]
			key = "yaml"

			def __init__(self):
				raise NotImplementedError("'PyYAML' or 'ruamel.yaml' package not available.")


class SerializerRegistry:
	"""
	Serializes and deserializes data for transfer to and from a REST API.

	:param default: The default serializer to use if none is specified.
		Corresponds to the :attr:`~.Serializer.key` of a :class:`~.Serializer`.
	:param serializers: List of :class:`~.Serializer` objects to use.

	.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`
	"""

	def __init__(self, default: str = "json", serializers: Optional[List[Serializer]] = None):

		#: Mapping of formats to :class:`~.Serializer` objects.
		self.serializers: Dict[str, Serializer] = {}

		for serializer in serializers or [x() for x in _SERIALIZERS]:
			self.serializers[serializer.key] = serializer

		#: The default serializer to use if none is specified.
		self.default: str = default

	def get_serializer(self, name: Optional[str] = None, content_type: Optional[str] = None):
		"""
		Returns the first :class:`~.Serializer` that supports either the given
		format or the given content type.

		:param name:
		:param content_type:
		"""  # noqa: D400

		if content_type is None:
			if name is None:
				return self.serializers[self.default]
			else:
				if name not in self.serializers:
					raise SerializerNotAvailable(name)
				return self.serializers[name]
		else:
			for x in self.serializers.values():
				for ctype in x.content_types:
					if content_type == ctype:
						return x

			raise SerializerNotAvailable(content_type)

	def loads(
			self,
			data: str,
			format: Optional[str] = None,  # noqa: A002  # pylint: disable=redefined-builtin
			) -> MutableMapping[str, Any]:
		"""
		Deserialize data of the given format.

		:param data:
		:param format: The serialization format to use.
		"""

		s = self.get_serializer(format)
		return s.loads(data)

	def dumps(
			self,
			data: Mapping[str, Any],
			format: Optional[str] = None,  # noqa: A002  # pylint: disable=redefined-builtin
			) -> str:
		"""
		Serialize data of the given format.

		:param data:
		:param format: The serialization format to use.
		"""

		s = self.get_serializer(format)
		return s.dumps(data)

	def get_content_type(
			self,
			format: Optional[str] = None,  # noqa: A002  # pylint: disable=redefined-builtin
			):
		"""
		Returns the content type for the serializer that supports the given format.

		:param format: The desired serialization format.
		"""

		s = self.get_serializer(format)
		return s.get_content_type()


class SerializerNotAvailable(SlumberBaseException):
	"""
	The chosen :class:`Serializer` is not available.

	.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`
	"""

	def __init__(self, content_type: str):
		super().__init__(f"No serializer available for {content_type!r}.")


Serializer.__module__ = JsonSerializer.__module__



#!/usr/bin/env python
#
#  _pyyaml_serializer.py
"""
ruamel.yaml-backed YAML serializer for :class:`~apeye.slumber_url.SlumberURL`.

.. versionadded:: 0.6.0
"""
#
#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#  Based on Slumber <https://slumber.readthedocs.io>
#  Copyright (c) 2011 Donald Stufft
#  Licensed under the 2-clause BSD License
#

# stdlib
from io import StringIO
from typing import Any, Mapping, MutableMapping

# 3rd party
import ruamel.yaml as yaml  # type: ignore[import]  # nodep

# this package
from apeye.slumber_url.serializers._abc import Serializer

__all__ = ["YamlSerializer"]


class YamlSerializer(Serializer):
	"""
	Serializer for YAML data.

	.. versionchanged:: 0.6.0  Moved to :mod:`apeye.slumber_url.serializers`

	.. attention::

		Either `PyYaml <https://pypi.org/project/PyYAML/>`_ or
		`ruamel.yaml <https://pypi.org/project/ruamel.yaml/>`_
		must be installed to use this serializer.
	"""

	content_types = ["text/yaml"]
	key = "yaml"

	def __init__(self):
		super().__init__()
		self._yaml = yaml.YAML()

	def loads(self, data: str) -> MutableMapping[str, Any]:
		"""
		Deserialize data using this :class:`~.Serializer`.

		:param data:
		"""

		y = yaml.YAML(typ="safe", pure=True)
		return y.load(str(data))

	def dumps(self, data: Mapping[str, Any]) -> str:
		"""
		Serialize data using this :class:`~.Serializer`.

		:param data:
		"""

		buf = StringIO()
		self._yaml.dump(data, buf)
		return buf.getvalue()



#!/usr/bin/env python
#
#  requests_url.py
"""
Extension of :class:`~apeye.url.URL` with support for interacting with the website using the :requests:`.` library.

.. versionadded:: 0.2.0
"""
#
#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
#  Some docstrings from Requests <https://requests.readthedocs.io>
#  Copyright 2019 Kenneth Reitz
#  Licensed under the Apache License, Version 2.0

# stdlib
import sys
from typing import IO, Any, Iterable, List, Mapping, MutableMapping, Optional, Tuple, TypeVar, Union

# 3rd party
import requests

# this package
from apeye.url import URL

__all__ = ["RequestsURL", "TrailingRequestsURL", "_R"]

_ParamsMappingValueType = Union[str, bytes, int, float, Iterable[Union[str, bytes, int, float]]]
# _Data = Union[None, str, bytes, MutableMapping[str, Any], Iterable[Tuple[str, Optional[str]]], IO]
_Data = Union[
	None,
	str,
	bytes,
	MutableMapping[str, Any],
	List[Tuple[str, Optional[str]]],
	Tuple[Tuple[str, Optional[str]]],
	IO
	]
_ParamsType = Union[
	Mapping[Union[str, bytes, int, float], _ParamsMappingValueType],
	Union[str, bytes],
	Tuple[Union[str, bytes, int, float], _ParamsMappingValueType],
	None
	]

_R = TypeVar("_R", bound="RequestsURL")


# Ignore the LGTM warning as the "session" attribute should **not** affect equality.
class RequestsURL(URL):  # lgtm [py/missing-equals]
	"""
	Extension of :class:`~apeye.url.URL` with support for interacting with the website using the :requests:`.` library.

	The :class:`requests.Session` used for this object -- and all objects created using the
	``/`` or ``.parent`` operations -- can be accessed using the :attr:`~.session` attribute.
	If desired, this can be replaced with a different session object, such as one using caching.

	:param url: The url to construct the :class:`~apeye.url.URL` object from.

	.. latex:vspace:: 5px

	.. versionchanged:: 0.3.0  The ``url`` parameter can now be a string or a :class:`~.URL`.

	.. versionchanged:: 1.1.0

		When a :class:`~.RequestsURL` object is deleted or garbage collected,
		the underlying :class:`requests.Session` object it only closed if no objects hold references to the session.
		This prevents the session object of a global object from being inadvertently closed
		when one of its children is garbage collected.

	.. latex:vspace:: -4px
	"""

	#: The underlying requests session.
	session: requests.Session

	def __init__(self, url: Union[str, URL] = ''):
		super().__init__(url)
		self.session = requests.Session()

	def resolve(self: _R, timeout: Optional[int] = None) -> _R:
		"""
		Resolves the URL into its canonical form.

		This is done by making a ``HEAD`` request and following HTTP 302 redirects.

		.. versionadded:: 0.8.0

		.. versionchanged:: 1.1.0  Added the ``timeout`` argument.
		"""

		response: requests.Response = self.head(allow_redirects=True, timeout=timeout)

		if response.status_code != 200:
			raise requests.HTTPError(f"Could not resolve {self!r}: HTTP Status {response.status_code}")

		new_obj = self.__class__(response.url)
		new_obj.session = self.session
		return new_obj

	def get(self, params: _ParamsType = None, **kwargs) -> requests.Response:
		r"""
		Perform a GET request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/GET

		:param params: Dictionary, list of tuples or bytes to send
			in the query string for the :class:`requests.Request`.
		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.

		.. versionchanged:: 0.7.0

			If ``params`` is :py:obj:`None` but the URL has a query string,
			the query string will be parsed and used for ``params``.
		"""

		if params is None and self.query:
			params = self.query  # type: ignore

		return self.session.get(str(self.base_url), params=params, **kwargs)

	def options(self, **kwargs) -> requests.Response:
		r"""
		Send an OPTIONS request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/OPTIONS

		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
		"""

		return self.session.options(str(self.base_url), **kwargs)

	def head(self, **kwargs) -> requests.Response:
		r"""
		Send a HEAD request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/HEAD

		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
			If `allow_redirects` is not provided, it will be set to `False`
			(as opposed to the default :func:`requests.request` behavior).
		"""

		return self.session.head(str(self.base_url), **kwargs)

	def post(self, data: "_Data" = None, json=None, **kwargs) -> requests.Response:
		r"""
		Send a POST request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/POST

		:param data: Dictionary, list of tuples, bytes, or file-like
			object to send in the body of the :class:`requests.Request`.
		:param json: json data to send in the body of the :class:`requests.Request`.
		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
		"""

		return self.session.post(str(self.base_url), data=data, json=json, **kwargs)

	def put(self, data: "_Data" = None, json=None, **kwargs) -> requests.Response:
		r"""
		Send a PUT request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/PUT

		:param data: Dictionary, list of tuples, bytes, or file-like
			object to send in the body of the :class:`requests.Request`.
		:param json: json data to send in the body of the :class:`requests.Request`.
		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
		"""

		return self.session.put(str(self.base_url), data=data, json=json, **kwargs)

	def patch(self, data: "_Data" = None, json=None, **kwargs) -> requests.Response:
		r"""
		Send a PATCH request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/PATCH

		:param data: Dictionary, list of tuples, bytes, or file-like
			object to send in the body of the :class:`requests.Request`.
		:param json: json data to send in the body of the :class:`requests.Request`.
		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
		"""

		return self.session.patch(str(self.base_url), data=data, json=json, **kwargs)

	def delete(self, **kwargs) -> requests.Response:
		r"""
		Send a DELETE request using :requests:`.`.

		https://developer.mozilla.org/en-US/docs/Web/HTTP/Methods/DELETE

		:param \*\*kwargs: Optional arguments that :func:`requests.request` takes.
		"""

		return self.session.delete(str(self.base_url), **kwargs)

	def __del__(self):  # pragma: no cover
		"""
		Attempt to close session when garbage collected to avoid leaving connections open.
		"""

		try:
			if sys.getrefcount(self.session) <= 2:
				self.session.close()
		except Exception:  # nosec: B110  # pylint: disable=bare-except
			pass

	def __truediv__(self, other):
		"""
		Construct a new :class:`~apeye.url.URL` object for the given child of this :class:`~apeye.url.URL`.
		"""

		new_obj = super().__truediv__(other)

		if new_obj is not NotImplemented:
			new_obj.session = self.session

		return new_obj


class TrailingRequestsURL(RequestsURL):
	"""
	Extension of :class:`~apeye.requests_url.RequestsURL` which adds a trailing slash to the end of the URL.

	.. versionadded:: 0.5.0

	:param url: The url to construct the :class:`~apeye.url.URL` object from.

	.. autoclasssumm:: TrailingRequestsURL
		:autosummary-sections: ;;
	"""

	def __str__(self) -> str:
		"""
		Returns the :class:`~.TrailingRequestsURL` as a string.
		"""

		return super().__str__().rstrip('/') + '/'



#!/usr/bin/env python
#
#  url.py
"""
Email address validation functions.

.. versionadded:: 1.0.0

This module is a subset of https://pypi.org/project/email-validator

.. note::

	The classes in this module can instead be imported from the
	`apeye_core.email_validator <https://pypi.org/project/apeye-core/>`_ module instead.
"""
#
#  Copyright © 2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# Based on https://github.com/JoshData/python-email-validator
# Licensed under the CC0 License
#

# stdlib
import sys

# 3rd party
from apeye_core import email_validator

__all__ = [
		"EmailSyntaxError",
		"ValidatedEmail",
		"main",
		"validate_email",
		"validate_email_domain_part",
		"validate_email_local_part"
		]

EmailSyntaxError = email_validator.EmailSyntaxError
ValidatedEmail = email_validator.ValidatedEmail
ValidatedEmail.__module__ = "apeye.email_validator"

validate_email = email_validator.validate_email
validate_email_local_part = email_validator.validate_email_local_part
validate_email_domain_part = email_validator.validate_email_domain_part

main = email_validator.main

if __name__ == "__main__":
	sys.exit(email_validator.main())



#!/usr/bin/env python
#
#  url.py
"""
:mod:`pathlib`-like approach to URLs.

.. versionchanged:: 0.2.0

	:class:`~apeye.slumber_url.SlumberURL` and :class:`~apeye.requests_url.RequestsURL`
	moved to :mod:`apeye.slumber_url` and :mod:`apeye.requests_url` respectively.

.. note::

	The classes in this module can instead be imported from the
	`apeye_core <https://pypi.org/project/apeye-core/>`_ module instead.
"""
#
#  Copyright © 2020-2021 Dominic Davis-Foster <dominic@davis-foster.co.uk>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#

# stdlib
import os
import sys
from typing import TYPE_CHECKING, Dict, List, Optional

# 3rd party
import apeye_core
from apeye_core import URL as URL
from apeye_core import Domain as Domain
from apeye_core import URLPath as URLPath
from apeye_core import URLPathType as URLPathType
from apeye_core import URLType as URLType

__all__ = ["URL", "URLPath", "Domain", "URLType", "URLPathType"]

if TYPE_CHECKING:
	pass
elif "_APEYE_DOCS" in os.environ:

	class URL(apeye_core.URL):  # noqa: D101
		#: URL scheme specifier
		scheme: str

		#: Network location part of the URL
		netloc: str

		#: The hierarchical path of the URL
		path: URLPath

		query: Dict[str, List[str]]
		"""
		The query parameters of the URL, if present.

		.. versionadded:: 0.7.0
		"""

		fragment: Optional[str]
		"""
		The URL fragment, used to identify a part of the document. :py:obj:`None` if absent from the URL.

		.. versionadded:: 0.7.0
		"""


URLPath.__module__ = "apeye.url"
URL.__module__ = "apeye.url"
Domain.__module__ = "apeye.url"

if sys.version_info >= (3, 7):
	URLType.__module__ = "apeye.url"
	URLPathType.__module__ = "apeye.url"
